@@ -25,6 +25,7 @@ import gst
 from StreamDecoder import StreamDecoder
 from lib.common import USER_AGENT
 from events.EventManager import EventManager
+from threading import Timer
 import logging
 
 class AudioPlayerGStreamer:
@@ -34,6 +35,7 @@ class AudioPlayerGStreamer:
         self.eventManager = eventManager
         self.decoder = StreamDecoder(cfg_provider)
         self.playlist = []
+        self.retrying = False
 
         self.log = logging.getLogger('radiotray')
 
@@ -44,7 +46,16 @@ class AudioPlayerGStreamer:
         self.player = gst.element_factory_make("playbin2", "player")		
         fakesink = gst.element_factory_make("fakesink", "fakesink")
         self.player.set_property("video-sink", fakesink)
-        #self.player.set_property("buffer-duration", 10000)
+
+        #buffer size
+        if(cfg_provider._settingExists("buffer_size")):
+            bufferSize = int(cfg_provider.getConfigValue("buffer_size"))
+            if (bufferSize > 0):
+            
+                self.log.debug("Setting buffer size to " + str(bufferSize))
+                self.player.set_property("buffer-size", bufferSize)
+
+        
         bus = self.player.get_bus()
         bus.add_signal_watch()
         bus.connect("message", self.on_message)
@@ -78,8 +89,7 @@ class AudioPlayerGStreamer:
 
             urlInfo = self.decoder.getMediaStreamInfo(stream)
             if(urlInfo is not None and urlInfo.isPlaylist() == False):
-                self.player.set_property("uri", stream)
-                self.player.set_state(gst.STATE_PAUSED) # buffer before starting playback
+                self.playStream(stream)
             elif(urlInfo is not None and urlInfo.isPlaylist()):
                 self.playlist = self.decoder.getPlaylist(urlInfo) + self.playlist
                 self.playNextStream()
@@ -90,6 +100,11 @@ class AudioPlayerGStreamer:
             self.eventManager.notify(EventManager.STATE_CHANGED, {'state':'paused'})
         self.mediator.updateVolume(self.player.get_property("volume"))
 
+
+    def playStream(self, uri):
+        self.player.set_property("uri", uri)
+        self.player.set_state(gst.STATE_PAUSED) # buffer before starting playback
+
     def stop(self):
         self.player.set_state(gst.STATE_NULL)
         self.eventManager.notify(EventManager.STATE_CHANGED, {'state':'paused'})
@@ -108,8 +123,8 @@ class AudioPlayerGStreamer:
         stru = message.structure
         if(stru != None):
             name = stru.get_name()
-            self.log.debug(name)
             if(name == 'redirect'):
+                slef.log.info("redirect received")
                 self.player.set_state(gst.STATE_NULL)
                 stru.foreach(self.redirect, None)
 
@@ -120,7 +135,9 @@ class AudioPlayerGStreamer:
             self.player.set_state(gst.STATE_NULL)
             self.playNextStream()
         elif t == gst.MESSAGE_BUFFERING:
-            if message.structure['buffer-percent'] == 0:
+            percent = message.structure['buffer-percent']
+            if percent < 100:
+                self.log.debug("Buffering %s" % percent)
                 self.player.set_state(gst.STATE_PAUSED)
             else:
                 self.player.set_state(gst.STATE_PLAYING)
@@ -137,27 +154,39 @@ class AudioPlayerGStreamer:
                 self.eventManager.notify(EventManager.STATION_ERROR, {'error':debug})
 
         elif t == gst.MESSAGE_STATE_CHANGED:
-            self.log.debug("Received MESSAGE_STATE_CHANGED")
             oldstate, newstate, pending = message.parse_state_changed()
+            self.log.debug(("Received MESSAGE_STATE_CHANGED (%s -> %s)") % (oldstate, newstate))
 
             if newstate == gst.STATE_PLAYING:
+                self.retrying = False
                 station = self.mediator.getContext().station
                 self.eventManager.notify(EventManager.STATE_CHANGED, {'state':'playing', 'station':station})
-            #elif newstate == gst.STATE_NULL:
-                #self.mediator.notifyStopped()
+            elif oldstate == gst.STATE_PLAYING and newstate == gst.STATE_PAUSED:
+                self.log.info("Received PAUSE state.")
+                
+                if self.retrying == False:
+                    self.retrying = True
+                    timer = Timer(20.0, self.checkTimeout)
+                    timer.start()
+                    self.eventManager.notify(EventManager.STATE_CHANGED, {'state':'paused'})
+                    
+            
 
         elif t == gst.MESSAGE_TAG:
 
            taglist = message.parse_tag()
-           station = self.mediator.getContext().station
-           metadata = {}
 
-           for key in taglist.keys():      
-               metadata[key] = taglist[key]
+           #if there is no song information, there's no point in triggering song change event
+           if('artist' in taglist.keys() or 'title' in taglist.keys()):
+               station = self.mediator.getContext().station
+               metadata = {}
 
-           metadata['station'] = station
+               for key in taglist.keys():      
+                   metadata[key] = taglist[key]
+
+               metadata['station'] = station
            
-           self.eventManager.notify(EventManager.SONG_CHANGED, metadata)
+               self.eventManager.notify(EventManager.SONG_CHANGED, metadata)
 
         return True
 
@@ -165,3 +194,13 @@ class AudioPlayerGStreamer:
         if(name == 'new-location'):
             self.start(value)
         return True
+
+
+    def checkTimeout(self):
+        self.log.debug("Checking timeout...")
+        if self.retrying == True:
+            self.log.info("Timed out. Retrying...")
+            uri = self.player.get_property("uri")
+            self.playStream(uri)
+        else: 
+            self.log.info("Timed out, but not retrying anymore")
