@@ -92,6 +92,7 @@ class StateMediator(object):
             self.play(self.context.station)
 
     def stop(self):
+        self.log.debug('Stopping...')
         self.audioPlayer.stop()
 
 
@@ -131,7 +132,7 @@ class StateMediator(object):
         
         
     def on_song_changed(self, data):
-        print data
+
         if('artist' in data.keys()):
             self.context.artist = data['artist']
         if('title' in data.keys()):
