@@ -7,7 +7,7 @@ from xdg.BaseDirectory import xdg_data_home
 try:
     from defs import *
 except:
-    APPVERSION = "0.7.2"
+    APPVERSION = "0.7.3"
     datadir = "/usr/share"
 
 # Application info
@@ -57,9 +57,9 @@ APP_ICON = os.path.join(IMAGE_PATH, 'radiotray.png')
 APP_ICON_ON = os.path.join(IMAGE_PATH, 'radiotray_on.png')
 APP_ICON_OFF = os.path.join(IMAGE_PATH, 'radiotray_off.png')
 APP_ICON_CONNECT = os.path.join(IMAGE_PATH, 'radiotray_connecting.gif')
-APP_INDICATOR_ICON_ON = "radiotray-on"
-APP_INDICATOR_ICON_OFF = "radiotray-off"
-
+APP_INDICATOR_ICON_ON = "radiotray_on"
+APP_INDICATOR_ICON_OFF = "radiotray_off"
+APP_INDICATOR_ICON_CONNECT = "radiotray_connecting"
 # Config info
 CFG_NAME = 'bookmarks.xml'
 OPTIONS_CFG_NAME = 'config.xml'
@@ -82,6 +82,9 @@ USER_PLUGIN_PATH = os.path.join(USER_CFG_PATH, 'plugins')
 #Logfile
 LOGFILE = os.path.join(USER_CFG_PATH,'radiotray.log')
 
+#temporary icon file
+ICON_FILE = os.path.join(USER_CFG_PATH,'icon')
+
 # user-agent
 try:
 	import platform
